#each language character set can be divided into for sets, m : modifiers like anuswar,  v : matras , V : independent
# vowels and CH: consonants.
# below configuration is for Hindi, similar configuration can be added for other languages.

symbols= ") + * / , . : ; = > ] - _ | ~ %"

HI_C = "क ख ग घ ङ च छ ज झ ञ ट ठ ड ढ ण त थ द ध न ऩ प फ ब भ म य र ऱ ल ळ ऴ व श ष स ह क़ ख़ ग़ ज़ ड़ ढ़ फ़ य़ ॸ ॹ ॺ ॻ ॼ ॽ ॾ ॿ"
HI_V = "ऄ अ आ इ ई उ ऊ ऋ ऌ ऍ ऎ ए ऐ ऑ ऒ ओ औ ॠ ॡ ॲ ॳ ॴ ॵ ॶ ॷ"
HI_v="ऺ ऻ ा ि ी ु ू ृ ॄ ॅ ॆ े ै ॉ ॊ ो ौ ॎ ॏ ॕ ॖ ॗ ॢ ॣ "
HI_m="ऀ ँ ं ः ऽ ़ "
HI_H="् "
HI_digits="०  १  २  ३  ४  ५  ६  ७  ८  ९"
HI_dont_care="। ॥ ॐ"     # TODO don't know what to do with dont cares. so right now keeping in the config file but not
# using it.
HI_symbols = HI_digits + symbols


BN_C = "ক খ গ ঘ ঙ চ ছ জ ঝ ঞ ট ঠ ড ঢ ণ ত থ দ ধ ন প ফ ব ভ ম য র ল শ ষ স হ ৎ ড় ঢ় য়ৰ ৱ "
BN_V="অ আ ই ঈ উ ঊ ঋ ঌ এ ঐ ও ঔ ৠ ৡ "
BN_v="া ি ী ু ূ ৃ ৄ ে ৈ ো ৌ ৗ ৢ ৣ "
BN_m="ঀ ঁ ং ঃ ঽ ় "
BN_H="্ "
BN_digits="০ ১ ২ ৩ ৪ ৫ ৬ ৭ ৮ ৯ "
BN_dont_care="৲ ৳ ৴ ৵ ৶ ৷ ৸ ৹ ৺ ৻ ৼ ৽ ৾ "
BN_symbols=BN_digits + symbols


GUR_C ="ਕ ਖ ਗ ਘ ਙ ਚ ਛ ਜ ਝ ਞ ਟ ਠ ਡ ਢ ਣ ਤ ਥ ਦ ਧ ਨ ਪ ਫ ਬ ਭ ਮ ਯ ਰ ਲ ਲ਼ ਵ ਸ਼ ਸ ਹ ਖ਼ ਗ਼ ਜ਼ ੜ ਫ਼ ੵ "
GUR_V="ਅ ਆ ਇ ਈ ਉ ਊ ਏ ਐ ਓ ਔ ੲ ੳ"
GUR_v="ਾ ਿ ੀ ੁ ੂ ੇ ੈ ੋ ੌ "
GUR_m="ਁ ਂ ੰ ਃ ਼ ੰ ੱ "
GUR_H="੍ "
GUR_digits= "੦  ੧  ੨  ੩  ੪  ੫  ੬  ੭  ੮  ੯"
GUR_dont_care= "ੴ ੶"
GUR_symbols=GUR_digits + symbols

GU_C="ક ખ ગ ઘ ઙ ચ છ જ ઝ ઞ ટ ઠ ડ ઢ ણ ત થ દ ધ ન પ ફ બ ભ મ ય ર લ ળ વ શ ષ સ હ ૹ"
GU_V="અ આ ઇ ઈ ઉ ઊ ઋ ઌ ઍ એ ઐ ઑ ণ ত ૠ ૡ "
GU_v="ા િ ી ુ ૂ ૃ ૄ ૅ ે ૈ ૉ ો ૌ ૢ ૣ "
GU_m="ઁ ં ઃ ઽ ઼ "
GU_H="્ "
GU_digits="૦  ૧  ૨  ૩  ૪  ૫  ૬  ૭  ૮  ૯ "
GU_dont_care="ઓ ઔ ૐ ૤ ૥ ૰ ૱"
GU_symbols=GU_digits + symbols


OR_C="କ ଖ ଗ ଘ ଙ ଚ ଛ ଜ ଝ ଞ ଟ ଠ ଡ ଢ ଣ ତ ଥ ଦ ଧ ନ ପ ଫ ବ ଭ ମ ଯ ର ଲ ଳ ଵ ଶ ଷ ସ ହ ଡ଼ ଢ଼ ୟ ୱ "
OR_V="ଅ ଆ ଇ ଈ ଉ ଊ ଋ ଌ ଏ ଐ ଓ ଔ ୠ ୡ "
OR_v="ା ି ୀ ୁ ୂ ୃ ୄ େ ୈ ୋ ୌ ୖ ୗ ୢ ୣ"
OR_m="ଁ ଂ ଃ ଽ ଼ "
OR_H="୍ "
OR_digits="୦  ୧  ୨  ୩  ୪  ୫  ୬  ୭  ୮  ୯"
OR_dont_care="଱ ୰ ୲ ୳ ୴ ୵ ୶ ୷ "
OR_symbols= OR_digits + symbols


ML_C='ക ഖ ഗ ഘ ങ ച ഛ ജ ഝ ഞ ട ഠ ഡ ഢ ണ ത ഥ ദ ധ ന ഩ പ ഫ ബ ഭ മ യ ര റ ല ള ഴ വ ശ ഷ സ ഹ ഺ ൺ ൻ ർ ൽ ൾ ൿ'
ML_V='അ ആ ഇ ഈ ഉ ഊ ഋ ഌ എ ഏ ഐ ഒ ഓ ഔ ൟ ൠ ൡ '
ML_v= 'ാ ി ീ ു ൂ ൃ ൄ െ േ ൈ ൊ ോ ൌ ൗ ൢ ൣ '
ML_m=' ഁ ം ഃ ഽ '
ML_H='് '
ML_digit='൦  ൧  ൨  ൩  ൪  ൫  ൬  ൭  ൮  ൯'
ML_symbols=symbols+ML_digit
ML_dont_care=' ഀ ഄ ഍ ഑ ഻ ഼ ൅ ൉ ൎ ൏ ൐ ൑ ൒ ൓ ൔ ൕ ൖ ൘ ൙ ൚ ൛ ൜ ൝ ൞ ൤ ൥ ൰ ൱ ൲ ൳ ൴ ൵ ൶ ൷ ൸ ൹ '

KN_C="ಕ ಖ ಗ ಘ ಙ ಚ ಛ ಜ ಝ ಞ ಟ ಠ ಡ ಢ ಣ ತ ಥ ದ ಧ ನ ಪ ಫ ಬ ಭ ಮ ಯ ರ ಱ ಲ ಳ ವ ಶ ಷ ಸ ಹ ೞ"
KN_V='ಅ ಆ ಇ ಈ ಉ ಊ ಋ ಌ ಎ ಏ ಐ ಒ ಓ ಔ ೠ ೡ '
KN_v='ಾ ಿ ೀ ು ೂ ೃ ೄ ೆ ೇ ೈ ೊ ೋ ೌ ೕ ೖ ೢ ೣ '
KN_m='ಂ ಃ ಼ ಽ ೱ ೲ '
KN_H='್ '
KN_digit='೦ ೧  ೨  ೩  ೪  ೫  ೬  ೭  ೮  ೯ '
KN_symbols=symbols + KN_digit
KN_dont_care='ಀ ಁ ಄ ೅ ೉'

TA_C='க ங ச ஜ ஞ ட ண த ந ன ப ம ய ர ற ல ள ழ வ ஶ ஷ ஸ ஹ'
TA_V='அ ஆ இ ஈ உ ஊ எ ஏ ஐ ஒ ஓ ஔ '
TA_v='ா ி ீ ு ூ ெ ே ை ொ ோ ௌ '
TA_m='ஂ ஃ '
TA_H='்'
TA_digit='௦  ௧  ௨  ௩  ௪  ௫  ௬  ௭  ௮  ௯ '
TA_symbols=symbols+TA_digit
TA_dont_care=' ஗  ஬  ஻ ஼ ஽  ௏ ௐ  ௗ ௘  ௛ ௞  ௰ ௱ ௲ ௳ ௴ ௵ ௶ ௷ ௸ ௹ ௺ '

TE_C='క ఖ గ ఘ ఙ చ ఛ జ ఝ ఞ ట ఠ డ ఢ ణ త థ ద ధ న ప ఫ బ భ మ య ర ఱ ల ళ ఴ వ శ ష స హ'
TE_V='అ ఆ ఇ ఈ ఉ ఊ ఋ ఌ ఎ ఏ ఐ ఒ ఓ ఔ ౠ ౡ'
TE_v='ా ి ీ ు ూ ృ ౄ ె ే ై ొ ో ౌ ౕ ౖ ౢ ౣ '
TE_m='ఀ ఁ ం ః ఽ '
TE_H='్ '
TE_digit='౦ ౧  ౨  ౩  ౪  ౫  ౬  ౭  ౮  ౯ '
TE_symbols=symbols+TE_digit
TE_dont_care='ౘ ౙ ౚ  ౷ ౸ ౹ ౺ ౻ ౼ ౽ ౾ ౿ '



import itertools
import abfn.configuration as configuration

def detect_lang(text):
	for c in list(text):
		if c in configuration.symbols:
			continue
		else:
			if "ऀ" <= c <= "ॿ":
				return "HI"
			
			if "ঀ" <= c <= "৾":
				return "BN"
			
			if "ઁ" <= c <= "૿":
				return "GU"
			
			if "ਁ" <= c <= "੶":
				return "GUR"
			
			if "ஂ" <= c <= "௺":
				return "TA"
			
			if "ఀ" <= c <= "౿":
				return "TE"
			
			if "ಀ" <= c <= "೯":
				return "KN"
			
			if "ഀ" <= c <= "ൿ":
				return "ML"
			
			if "ଁ" <= c <= "୷":
				return "OR"
			
			if "؀" <= c <= "ۿ":
				return "AR"
			
			if "!" <= c <= "~":
				return "EN"
			
			else:
				print("unknown language !!! text: {}".format(c))
				return None


def get_configuration_based_on_lang(text):
	lang = detect_lang(text)
	if lang == "HI":
		return (configuration.HI_m, configuration.HI_v, configuration.HI_V, configuration.HI_C, \
		        configuration.HI_symbols, configuration.HI_H)
	
	if lang == "GUR":
		return (configuration.GUR_m, configuration.GUR_v, configuration.GUR_V, configuration.GUR_C, \
		        configuration.GUR_symbols, configuration.GUR_H)
	if lang == "GU":
		return (configuration.GU_m, configuration.GU_v, configuration.GU_V, configuration.GU_C, \
		        configuration.GU_symbols, configuration.GU_H)
	
	if lang == "BN":
		return (configuration.BN_m, configuration.BN_v, configuration.BN_V, configuration.BN_C, \
		        configuration.BN_symbols, configuration.BN_H)
	
	if lang == "TE":
		return (configuration.TE_m, configuration.TE_v, configuration.TE_V, configuration.TE_C, \
		        configuration.TE_symbols, configuration.TE_H)
	
	if lang == "TA":
		return (configuration.TA_m, configuration.TA_v, configuration.TA_V, configuration.TA_C, \
		        configuration.TA_symbols, configuration.TA_H)
	
	if lang == "ML":
		return (configuration.ML_m, configuration.ML_v, configuration.ML_V, configuration.ML_C, \
		        configuration.ML_symbols, configuration.ML_H)
	
	if lang == "KN":
		return (configuration.KN_m, configuration.KN_v, configuration.KN_V, configuration.KN_C, \
		        configuration.KN_symbols, configuration.KN_H)
	
	if lang == "OR":
		return (configuration.OR_m, configuration.OR_v, configuration.OR_V, configuration.OR_C, \
		        configuration.OR_symbols, configuration.OR_H)
	
	return None


def print_unicode_characters_in_range(a, b):
	s = ""
	ch = a
	while (ch <= b):
		s += ch
		s += " "
		# print(ch, end=" ")
		ch = chr(ord(ch) + 1)
	
	return s


def get_unicode_value(ch):
	return ord(ch)


def print_in_range(xx):
	str = ""
	for x in xx:
		if "." in x:
			s = x.replace(".", "")
			s = [s[:4], s[4:]]
			s[0] = chr(int(s[0], 16))
			s[1] = chr(int(s[1], 16))
		else:
			s = []
			s.append(chr(int(x, 16)))
			s.append(chr(int(x, 16)))
		
		str += print_unicode_characters_in_range(s[0], s[1])
	print("")
	print(str)


def print_unused_char(a, b):
	from configuration import TE_symbols, TE_C, TE_H, TE_m, TE_V, TE_v
	ch = a
	while (ch <= b):
		if ch not in (TE_V + TE_v + TE_m + TE_H + TE_C + TE_symbols):
			print(ch, end=" ")
		ch = chr(ord(ch) + 1)


class ABFN:
	def is_valid_label(self, text):
		state = 0
		valid = True
		out = get_configuration_based_on_lang(text)
		
		if not out:
			print("text filtering rule not defined for give text language")
			return True
		
		m, v, V, C, symbols,H = out
		
		for ch in list(text):
			
			if not (ch in list(itertools.chain.from_iterable(out))):
				return False
			
			if ch in symbols:
				state = 0
				continue
			
			if ch in C:
				state = 2
				continue
			
			if ch in V:
				state = 1
				continue
			
			if state == 0:
				if ch in v or ch in m or ch in H:
					valid = False
					break
					
			if state == 1:
				if ch in v or ch in H:
					valid = False
					break
				
				if ch in m:
					state = 0
					continue
			
			if state == 2:
				if ch in v:
					state = 3
					continue
				
				if ch in m:
					state = 0
					continue
				
				if ch in H:
					state = 4
					continue
				
			if state == 3:
				if ch in m:
					state = 0
					continue
				
				if ch in v or ch in H:
					valid = False
					break
					
			if state ==4 :
				if ch in C:
					state = 2
					continue
				else:
					valid=False
					break
				
		return valid
	

if __name__ == "__main__":
	
	a = ABFN()
	s= ["/home/shubham/Documents/MTP/SyntheticDetectionTextGenration/SynthText/data/newsgroup/newsgroup_KN.txt"]
	for x in s:
		valid_cnt = 0
		invalid_count = 0
		
		with open(x) as f:
			for i , line in enumerate(f):
				if (i+1)%1000 ==0:
					print(i*100," ", "valid :", valid_cnt , "invalid: ", invalid_count)
					#break
					
				for word in line.split(" "):
					word=word.strip()
					if a.is_valid_label(word):
						valid_cnt+=1
					else:
						invalid_count+=1
						#print(word)
		
		print("invalid: ", invalid_count)
		print("valid: ",valid_cnt)
	
